@@ -1,17 +1,19 @@
 package Data::Printer;
 use strict;
 use warnings;
-use Term::ANSIColor;
+use Term::ANSIColor qw(color colored);
 use Scalar::Util;
 use Sort::Naturally;
 use Carp qw(croak);
-use Clone qw(clone);
-use Hash::FieldHash qw(fieldhash);
+use Clone::PP qw(clone);
+use if $] >= 5.010, 'Hash::Util::FieldHash' => qw(fieldhash);
+use if $] < 5.010, 'Hash::Util::FieldHash::Compat' => qw(fieldhash);
 use File::Spec;
 use File::HomeDir ();
 use Fcntl;
+use version 0.77 ();
 
-our $VERSION = 0.29;
+our $VERSION = '0.31';
 
 BEGIN {
     if ($^O =~ /Win32/i) {
@@ -32,10 +34,16 @@ my $properties = {
     'sort_keys'      => 1,
     'deparse'        => 0,
     'hash_separator' => '   ',
+    'separator'      => ',',
+    'end_separator'  => 0,
     'show_tied'      => 1,
     'show_tainted'   => 1,
     'show_weak'      => 1,
-    'escape_chars'   => 1,
+    'show_readonly'  => 0,
+    'show_lvalue'    => 1,
+    #'escape_chars'   => 1, ### <== DEPRECATED!!!
+    'print_escapes'  => 0,
+    'quote_keys'     => 'auto',
     'use_prototypes' => 1,
     'output'         => 'stderr',
     'return_value'   => 'dump',       # also 'void' or 'pass'
@@ -48,21 +56,27 @@ my $properties = {
         'number'      => 'bright_blue',
         'string'      => 'bright_yellow',
         'class'       => 'bright_green',
+        'method'      => 'bright_green',
         'undef'       => 'bright_red',
         'hash'        => 'magenta',
         'regex'       => 'yellow',
         'code'        => 'green',
         'glob'        => 'bright_cyan',
+        'vstring'     => 'bright_blue',
+        'lvalue'      => 'bright_white',
+        'format'      => 'bright_cyan',
         'repeated'    => 'white on_red',
         'caller_info' => 'bright_cyan',
         'weak'        => 'cyan',
         'tainted'     => 'red',
         'escaped'     => 'bright_red',
+        'unknown'     => 'bright_yellow on_blue',
     },
     'class' => {
         inherited    => 'none',   # also 'all', 'public' or 'private'
+        universal    => 1,
         parents      => 1,
-        linear_isa   => 1,
+        linear_isa   => 'auto',
         expand       => 1,        # how many levels to expand. 0 for none, 'all' for all
         internals    => 1,
         export       => 1,
@@ -72,14 +86,21 @@ my $properties = {
         _depth       => 0,        # used internally
     },
     'filters' => {
-        SCALAR => [ \&SCALAR ],
-        ARRAY  => [ \&ARRAY  ],
-        HASH   => [ \&HASH   ],
-        REF    => [ \&REF    ],
-        CODE   => [ \&CODE   ],
-        GLOB   => [ \&GLOB   ],
-        Regexp => [ \&Regexp ],
-        -class => [ \&_class ],
+        # The IO ref type isn't supported as you can't actually create one,
+        # any handle you make is automatically blessed into an IO::* object,
+        # and those are separately handled.
+        SCALAR  => [ \&SCALAR   ],
+        ARRAY   => [ \&ARRAY    ],
+        HASH    => [ \&HASH     ],
+        REF     => [ \&REF      ],
+        CODE    => [ \&CODE     ],
+        GLOB    => [ \&GLOB     ],
+        VSTRING => [ \&VSTRING  ],
+        LVALUE  => [ \&LVALUE ],
+        FORMAT  => [ \&FORMAT ],
+        Regexp  => [ \&Regexp   ],
+        -unknown=> [ \&_unknown ],
+        -class  => [ \&_class   ],
     },
 
     _output          => *STDERR,     # used internally
@@ -103,32 +124,7 @@ sub import {
     # the RC file overrides the defaults,
     # (and we load it only once)
     unless( exists $properties->{_initialized} ) {
-        my $file = ( $args && exists $args->{rc_file} )
-                 ? $args->{rc_file}
-                 : File::Spec->catfile(File::HomeDir->my_home,'.dataprinter')
-                 ;
-
-        if (-e $file) {
-            if ( open my $fh, '<', $file ) {
-                my $rc_data;
-                { local $/; $rc_data = <$fh> }
-                close $fh;
-
-                my $config = eval $rc_data;
-                if ( $@ ) {
-                    warn "Error loading $file: $@\n";
-                }
-                elsif (!ref $config or ref $config ne 'HASH') {
-                    warn "Error loading $file: config file must return a hash reference\n";
-                }
-                else {
-                    $properties = _merge( $config );
-                }
-            }
-            else {
-                warn "error opening '$file': $!\n";
-            }
-        }
+        _load_rc_file($args);
         $properties->{_initialized} = 1;
     }
 
@@ -179,7 +175,7 @@ sub _print_and_return {
         elsif ($ref eq 'HASH') {
             return %{ $item };
         }
-        elsif ( grep { $ref eq $_ } qw(REF SCALAR CODE Regexp GLOB) ) {
+        elsif ( grep { $ref eq $_ } qw(REF SCALAR CODE Regexp GLOB VSTRING) ) {
             return $$item;
         }
         else {
@@ -278,9 +274,21 @@ sub _p {
         }
     }
 
-    if (not $found) {
+    if (not $found and Scalar::Util::blessed($item) ) {
         # let '-class' filters have a go
         foreach my $filter ( @{ $p->{filters}->{'-class'} } ) {
+            if ( defined (my $result = $filter->($item, $p)) ) {
+                $string .= $result;
+                $found = 1;
+                last;
+            }
+        }
+    }
+    
+    if ( not $found ) {
+        # if it's not a class and not a known core type, we must be in
+        # a future perl with some type we're unaware of
+        foreach my $filter ( @{ $p->{filters}->{'-unknown'} } ) {
             if ( defined (my $result = $filter->($item, $p)) ) {
                 $string .= $result;
                 last;
@@ -312,30 +320,7 @@ sub SCALAR {
         $string .= colored($$item, $p->{color}->{'number'});
     }
     else {
-        my $val       = $$item;
-        my $str_color = color($p->{color}{string} );
-        my $esc_color = color($p->{color}{escaped});
-
-
-        unless ($p->{escape_chars}) {
-            my %escaped = (
-                    "\n" => $esc_color . '\n' . $str_color,
-                    "\r" => $esc_color . '\r' . $str_color,
-                    "\t" => $esc_color . '\t' . $str_color,
-                    "\f" => $esc_color . '\f' . $str_color,
-                    "\b" => $esc_color . '\b' . $str_color,
-                    "\a" => $esc_color . '\a' . $str_color,
-                    "\e" => $esc_color . '\e' . $str_color,
-            );
-            foreach my $k ( keys %escaped ) {
-                my $esc = $escaped{$k};
-                $val =~ s/$k/$esc/g;
-            }
-        }
-
-        # always escape the null character
-        my $null = $esc_color . '\0' . $str_color;
-        $val =~ s/\0/$null/g;
+        my $val = _escape_chars($$item, $p->{color}{string}, $p);
 
         $string .= colored(qq["$val"], $p->{color}->{'string'});
     }
@@ -345,9 +330,47 @@ sub SCALAR {
 
     $p->{_tie} = ref tied $$item;
 
+    if ($p->{show_readonly} and &Internals::SvREADONLY( $item )) {
+        $string .= ' (read-only)';
+    }
+
     return $string;
 }
 
+sub _escape_chars {
+    my ($str, $orig_color, $p) = @_;
+
+    $orig_color   = color( $orig_color );
+    my $esc_color = color( $p->{color}{escaped} );
+
+    my $escape_chars = 1;
+    if (exists $p->{escape_chars}) {
+        Carp::carp q('escape_chars' is deprecated!);
+        $escape_chars = $p->{escape_chars};
+    }
+
+    if ($p->{print_escapes} || !$escape_chars) {
+        my %escaped = (
+            "\n" => $esc_color . '\n' . $orig_color,
+            "\r" => $esc_color . '\r' . $orig_color,
+            "\t" => $esc_color . '\t' . $orig_color,
+            "\f" => $esc_color . '\f' . $orig_color,
+            "\b" => $esc_color . '\b' . $orig_color,
+            "\a" => $esc_color . '\a' . $orig_color,
+            "\e" => $esc_color . '\e' . $orig_color,
+        );
+        foreach my $k ( keys %escaped ) {
+            my $esc = $escaped{$k};
+            $str =~ s/$k/$esc/g;
+        }
+    }
+    # always escape the null character
+    my $null = $esc_color . '\0' . $orig_color;
+    $str =~ s/\0/$null/g;
+
+    return $str;
+}
+
 
 sub ARRAY {
     my ($item, $p) = @_;
@@ -389,7 +412,11 @@ sub ARRAY {
             $string .= ' ' . colored('(weak)', $p->{color}->{'weak'})
                 if $ref and Scalar::Util::isweak($item->[$i]) and $p->{show_weak};
 
-            $string .= ($i == $#{$item} ? '' : ',') . $BREAK;
+            $string .= $p->{separator}
+              if $i < $#{$item} || $p->{end_separator};
+
+            $string .= $BREAK;
+
             my $size = 2 + length($i); # [10], [100], etc
             substr $p->{name}, -$size, $size, '';
         }
@@ -451,26 +478,55 @@ sub HASH {
         $string .= "{$BREAK";
         $p->{_current_indent} += $p->{indent};
 
-        # length of the largest key is used for indenting
-        my $len = 0;
-        if ($p->{multiline}) {
-            foreach (keys %$item) {
-                my $l = length;
+        my $total_keys  = scalar keys %$item;
+        my $len         = 0;
+        my $multiline   = $p->{multiline};
+        my $hash_color  = $p->{color}{hash};
+        my $quote_keys  = $p->{quote_keys};
+
+        my @keys = ();
+
+        # first pass, preparing keys to display (and getting largest key size)
+        foreach my $key ($p->{sort_keys} ? nsort keys %$item : keys %$item ) {
+            my $new_key = _escape_chars($key, $hash_color, $p);
+            my $colored = colored( $new_key, $hash_color );
+
+            # wrap in uncolored single quotes if there's
+            # any space or escaped characters
+            if ( $quote_keys
+                  and (
+                        $quote_keys ne 'auto'
+                        or (
+                             $key eq q()
+                             or $new_key ne $key
+                             or $new_key =~ /\s|\n|\t|\r/
+                        )
+                  )
+            ) {
+                $colored = qq['$colored'];
+            }
+
+            push @keys, {
+                raw     => $key,
+                colored => $colored,
+            };
+
+            # length of the largest key is used for indenting
+            if ($multiline) {
+                my $l = length $colored;
                 $len = $l if $l > $len;
             }
         }
 
-        my $total_keys = scalar keys %$item;
-        my @keys = ($p->{sort_keys} ? nsort keys %$item : keys %$item );
+        # second pass, traversing and rendering
         foreach my $key (@keys) {
-            $p->{name} .= "{$key}";
-            my $element = $item->{$key};
+            my $raw_key     = $key->{raw};
+            my $colored_key = $key->{colored};
+            my $element     = $item->{$raw_key};
+            $p->{name}     .= "{$raw_key}";
 
             $string .= (' ' x $p->{_current_indent})
-                     . colored(
-                             sprintf("%-*s", $len, $key),
-                             $p->{color}->{'hash'}
-                       )
+                     . sprintf("%-*s", $len, $colored_key)
                      . $p->{hash_separator}
                      ;
 
@@ -483,12 +539,18 @@ sub HASH {
             else {
                 $string .= _p( $element, $p );
             }
+
             $string .= ' ' . colored('(weak)', $p->{color}->{'weak'})
-                if $ref and Scalar::Util::isweak($item->{$key}) and $p->{show_weak};
+                if $ref
+                  and $p->{show_weak}
+                  and Scalar::Util::isweak($item->{$raw_key});
 
-            $string .= (--$total_keys == 0 ? '' : ',') . $BREAK;
+            $string .= $p->{separator}
+              if --$total_keys > 0 || $p->{end_separator};
 
-            my $size = 2 + length($key); # {foo}, {z}, etc
+            $string .= $BREAK;
+
+            my $size = 2 + length($raw_key); # {foo}, {z}, etc
             substr $p->{name}, -$size, $size, '';
         }
         $p->{_current_indent} -= $p->{indent};
@@ -522,6 +584,28 @@ sub Regexp {
     return $string;
 }
 
+sub VSTRING {
+    my ($item, $p) = @_;
+    my $string = '';
+    $string .= colored(version->declare($$item)->normal, $p->{color}->{'vstring'});
+    return $string;
+}
+
+sub FORMAT {
+    my ($item, $p) = @_;
+    my $string = '';
+    $string .= colored("FORMAT", $p->{color}->{'format'});
+    return $string;
+}
+
+sub LVALUE {
+    my ($item, $p) = @_;
+    my $string = SCALAR( $item, $p );
+    $string .= colored( ' (LVALUE)', $p->{color}{lvalue} )
+        if $p->{show_lvalue};
+
+    return $string;
+}
 
 sub GLOB {
     my ($item, $p) = @_;
@@ -535,7 +619,7 @@ sub GLOB {
     # implement some of these flags (maybe not even
     # fcntl() itself, so we must wrap it.
     my $flags;
-    eval { $flags = fcntl($$item, F_GETFL, 0) };
+    eval { no warnings qw( unopened closed ); $flags = fcntl($$item, F_GETFL, 0) };
     if ($flags) {
         $extra .= ($flags & O_WRONLY) ? 'write-only'
                 : ($flags & O_RDWR)   ? 'read/write'
@@ -548,7 +632,7 @@ sub GLOB {
         # Solaris, for example, doesn't have O_ASYNC :(
         my %flags = ();
         eval { $flags{'append'}      = O_APPEND   };
-        eval { $flags{'async'}       = O_ASYNC    };
+        eval { $flags{'async'}       = O_ASYNC    }; # leont says this is the only one I should care for.
         eval { $flags{'create'}      = O_CREAT    };
         eval { $flags{'truncate'}    = O_TRUNC    };
         eval { $flags{'nonblocking'} = O_NONBLOCK };
@@ -559,7 +643,7 @@ sub GLOB {
         $extra .= ', ';
     }
     my @layers = ();
-    eval { @layers = PerlIO::get_layers $$item };
+    eval { @layers = PerlIO::get_layers $$item }; # TODO: try PerlIO::Layers::get_layers (leont)
     unless ($@) {
         $extra .= "layers: @layers";
     }
@@ -570,14 +654,22 @@ sub GLOB {
 }
 
 
+sub _unknown {
+    my($item, $p) = @_;
+    my $ref = ref $item;
+    
+    my $string = '';
+    $string = colored($ref, $p->{color}->{'unknown'});
+    return $string;
+}
+
 sub _class {
     my ($item, $p) = @_;
     my $ref = ref $item;
 
     # if the user specified a method to use instead, we do that
-    if ( $p->{class_method} and $item->can($p->{class_method}) ) {
-        my $method = $p->{class_method};
-        return $item->$method;
+    if ( $p->{class_method} and my $method = $item->can($p->{class_method}) ) {
+        return $method->($item, $p);
     }
 
     my $string = '';
@@ -599,10 +691,16 @@ sub _class {
 
         $p->{_current_indent} += $p->{indent};
 
-        require Class::MOP;
-        my $meta = Class::MOP::Class->initialize($ref);
+        if ($] >= 5.010) {
+            require mro;
+        } else {
+            require MRO::Compat;
+        }
+        require Package::Stash;
 
-        if ( my @superclasses = $meta->superclasses ) {
+        my $stash = Package::Stash->new($ref);
+
+        if ( my @superclasses = @{$stash->get_symbol('@ISA')||[]} ) {
             if ($p->{class}{parents}) {
                 $string .= (' ' x $p->{_current_indent})
                         . 'Parents       '
@@ -611,16 +709,22 @@ sub _class {
                         ) . $BREAK;
             }
 
-            if ($p->{class}{linear_isa}) {
+            if ( $p->{class}{linear_isa} and
+                  (
+                    ($p->{class}{linear_isa} eq 'auto' and @superclasses > 1)
+                    or
+                    ($p->{class}{linear_isa} ne 'auto')
+                  )
+            ) {
                 $string .= (' ' x $p->{_current_indent})
                         . 'Linear @ISA   '
                         . join(', ', map { colored( $_, $p->{color}->{'class'}) }
-                                  $meta->linearized_isa
+                                  @{mro::get_linear_isa($ref)}
                         ) . $BREAK;
             }
         }
 
-        $string .= _show_methods($ref, $meta, $p)
+        $string .= _show_methods($ref, $p)
             if $p->{class}{show_methods} and $p->{class}{show_methods} ne 'none';
 
         if ( $p->{'class'}->{'internals'} ) {
@@ -664,7 +768,7 @@ sub _class {
 
 
 sub _show_methods {
-    my ($ref, $meta, $p) = @_;
+    my ($ref, $p) = @_;
 
     my $string = '';
     my $methods = {
@@ -673,15 +777,41 @@ sub _show_methods {
     };
     my $inherited = $p->{class}{inherited} || 'none';
 
+    require B;
+
+    my $methods_of = sub {
+        my ($name) = @_;
+        map {
+            my $m;
+            if ($_
+                and $m = B::svref_2object($_)
+                and $m->isa('B::CV')
+                and not $m->GV->isa('B::Special')
+            ) {
+                [ $m->GV->STASH->NAME, $m->GV->NAME ]
+            } else {
+                ()
+            }
+        } values %{Package::Stash->new($name)->get_all_symbols('CODE')}
+    };
+
+    my %seen_method_name;
+
 METHOD:
-    foreach my $method ($meta->get_all_methods) {
-        my $method_string = $method->name;
+    foreach my $method (
+        map $methods_of->($_), @{mro::get_linear_isa($ref)},
+                               $p->{class}{universal} ? 'UNIVERSAL' : ()
+    ) {
+        my ($package_string, $method_string) = @$method;
+
+        next METHOD if $seen_method_name{$method_string}++;
+
         my $type = substr($method_string, 0, 1) eq '_' ? 'private' : 'public';
 
-        if ($method->package_name ne $ref) {
+        if ($package_string ne $ref) {
             next METHOD unless $inherited ne 'none'
                            and ($inherited eq 'all' or $type eq $inherited);
-            $method_string .= ' (' . $method->package_name . ')';
+            $method_string .= ' (' . $package_string . ')';
         }
 
         push @{ $methods->{$type} }, $method_string;
@@ -698,7 +828,7 @@ METHOD:
         $string .= (' ' x $p->{_current_indent})
                  . "$type methods (" . scalar @list . ')'
                  . (@list ? ' : ' : '')
-                 . join(', ', map { colored($_, $p->{color}->{class}) }
+                 . join(', ', map { colored($_, $p->{color}->{method}) }
                               @list
                    ) . $BREAK;
     }
@@ -740,11 +870,12 @@ sub _merge {
         foreach my $key (keys %$p) {
             if ($key eq 'color' or $key eq 'colour') {
                 my $color = $p->{$key};
-                if (defined $color and not $color) {
+                if ( not ref $color or ref $color ne 'HASH' ) {
+                    Carp::carp q['color' should be a HASH reference. Did you mean 'colored'?];
                     $clone->{color} = {};
                 }
                 else {
-                    foreach my $target ( keys %{$p->{$key}} ) {
+                    foreach my $target ( keys %$color ) {
                         $clone->{color}->{$target} = $p->{$key}->{$target};
                     }
                 }
@@ -831,6 +962,65 @@ sub _merge {
 }
 
 
+sub _load_rc_file {
+    my $args = shift || {};
+
+    my $file = exists $args->{rc_file}    ? $args->{rc_file}
+             : exists $ENV{DATAPRINTERRC} ? $ENV{DATAPRINTERRC}
+             : File::Spec->catfile(File::HomeDir->my_home,'.dataprinter');
+
+    return unless -e $file;
+
+    my $mode = (stat $file )[2];
+    if ($^O !~ /Win32/i && ($mode & 0020 || $mode & 0002) ) {
+        warn "rc file '$file' must NOT be writeable to other users. Skipping.\n";
+        return;
+    }
+
+    if ( -l $file || (!-f _) || -p _ || -S _ || -b _ || -c _ ) {
+        warn "rc file '$file' doesn't look like a plain file. Skipping.\n";
+        return;
+    }
+
+    unless (-o $file) {
+        warn "rc file '$file' must be owned by your (effective) user. Skipping.\n";
+        return;
+    }
+
+    if ( open my $fh, '<', $file ) {
+        my $rc_data;
+        { local $/; $rc_data = <$fh> }
+        close $fh;
+
+        if( ${^TAINT} != 0 ) {
+            if ( $args->{allow_tainted} ) {
+                warn "WARNING: Reading tainted file '$file' due to user override.\n";
+                $rc_data =~ /(.+)/s; # very bad idea - god help you
+                $rc_data = $1;
+            }
+            else {
+                warn "taint mode on: skipping rc file '$file'.\n";
+                return;
+            }
+        }
+
+        my $config = eval $rc_data;
+        if ( $@ ) {
+            warn "Error loading $file: $@\n";
+        }
+        elsif (!ref $config or ref $config ne 'HASH') {
+            warn "Error loading $file: config file must return a hash reference\n";
+        }
+        else {
+            $properties = _merge( $config );
+        }
+    }
+    else {
+        warn "error opening '$file': $!\n";
+    }
+}
+
+
 1;
 __END__
 
@@ -1039,16 +1229,21 @@ Note that both spellings ('color' and 'colour') will work.
         number      => 'bright_blue',   # numbers
         string      => 'bright_yellow', # strings
         class       => 'bright_green',  # class names
+        method      => 'bright_green',  # method names
         undef       => 'bright_red',    # the 'undef' value
         hash        => 'magenta',       # hash keys
         regex       => 'yellow',        # regular expressions
         code        => 'green',         # code references
         glob        => 'bright_cyan',   # globs (usually file handles)
+        vstring     => 'bright_blue',   # version strings (v5.16.0, etc)
         repeated    => 'white on_red',  # references to seen values
         caller_info => 'bright_cyan',   # details on what's being printed
         weak        => 'cyan',          # weak references
         tainted     => 'red',           # tainted content
         escaped     => 'bright_red',    # escaped characters (\t, \n, etc)
+
+        # potential new Perl datatypes, unknown to Data::Printer
+        unknown     => 'bright_yellow on_blue',
      },
    };
 
@@ -1104,7 +1299,14 @@ customization options available, as shown below (with default values):
       show_tied      => 1,       # expose tied variables
       show_tainted   => 1,       # expose tainted variables
       show_weak      => 1,       # expose weak references
-      escape_chars   => 0,       # escape non-printable chars (\n, \t, etc)
+      show_readonly  => 0,       # expose scalar variables marked as read-only
+      show_lvalue    => 1,       # expose lvalue types
+      print_escapes  => 0,       # print non-printable chars as "\n", "\t", etc.
+      quote_keys     => 'auto',  # quote hash keys (1 for always, 0 for never).
+                                 # 'auto' will quote when key is empty/space-only.
+      separator      => ',',     # uses ',' to separate array/hash elements
+      end_separator  => 0,       # prints the separator after last element in array/hash.
+                                 # the default is 0 that means not to print
 
       caller_info    => 0,       # include information on what's being printed
       use_prototypes => 1,       # allow p(%foo), but prevent anonymous data
@@ -1121,8 +1323,12 @@ customization options available, as shown below (with default values):
           inherited  => 'none',  # show inherited methods,
                                  # can also be 'all', 'private', or 'public'.
 
-          parents    => 1,       # show parents?
-          linear_isa => 1,       # show the entire @ISA, linearized
+          universal  => 1,       # include UNIVERSAL methods in inheritance list
+
+          parents    => 1,       # show parents, if there are any
+          linear_isa => 'auto',  # show the entire @ISA, linearized, whenever
+                                 # the object has more than one parent. Can
+                                 # also be set to 1 (always show) or 0 (never).
 
           expand     => 1,       # how deep to traverse the object (in case
                                  # it contains other objects). Defaults to
@@ -1138,6 +1344,15 @@ customization options available, as shown below (with default values):
 
 Note: setting C<multiline> to C<0> will also set C<index> and C<indent> to C<0>.
 
+=head3 WARNING: C<escape_chars> is **deprecated**:
+
+In versions 0.28 and 0.29 there was a property called 'escape_chars' that
+was replaced by 'print_escapes' to avoid ambiguity. The old name was
+confusing because 'escape' could be interpreted as a noun or as an adjective.
+
+It will still work until version 0.32, but wil trigger a warning so you
+can update your code and/or RC file. Please use 'print_escapes' instead. Thanks!
+
 
 =head1 FILTERS
 
@@ -1262,6 +1477,59 @@ while debugging scripts is:
 
 and it will load your custom settings every time :)
 
+=head2 Loading RC files in custom locations
+
+If your RC file is somewhere other than C<.dataprinter> in your home
+dir, you can load whichever file you want via the C<'rc_file'> parameter:
+
+  use Data::Printer rc_file => '/path/to/my/rcfile.conf';
+
+You can even set this to undef or to a non-existing file to disable your
+RC file at will.
+
+The RC file location can also be specified with the C<DATAPRINTERRC>
+environment variable. Using C<rc_file> in code will override the environment
+variable.
+
+=head2 RC File Security
+
+The C<.dataprinter> RC file is nothing but a Perl hash that
+gets C<eval>'d back into the code. This means that whatever
+is in your RC file B<WILL BE INTERPRETED BY PERL AT RUNTIME>.
+This can be quite worrying if you're not the one in control
+of the RC file.
+
+For this reason, Data::Printer takes extra precaution before
+loading the file:
+
+=over 4
+
+=item * The file has to be in your home directory unless you
+specifically point elsewhere via the 'C<rc_file>' property or
+the DATAPRINTERRC environment variable;
+
+=item * The file B<must> be a plain file, never a symbolic
+link, named pipe or socket;
+
+=item * The file B<must> be owned by you (i.e. the effective
+user id that ran the script using Data::Printer);
+
+=item * The file B<must> be read-only for everyone but your user.
+This usually means permissions C<0644>, C<0640> or C<0600> in
+Unix-like systems. B<THIS IS NOT CHECKED IN WIN32>;
+
+=item * The file will B<NOT> be loaded in Taint mode, unless
+you specifically load Data::Printer with the 'allow_tainted'
+option set to true. And even if you do that, Data::Printer
+will still issue a warning before loading the file. But
+seriously, don't do that.
+
+=back
+
+Failure to comply with the security rules above will result in
+the RC file not being loaded (likely with a warning on what went
+wrong).
+
 
 =head1 THE "DDP" PACKAGE ALIAS
 
@@ -1576,6 +1844,27 @@ In the example above, the C<$html_output> variable contains the
 HTML escaped output of C<p($object)>, so you can print it for
 later inspection or render it (if it's a web app).
 
+=head2 Using Data::Printer with Template Toolkit
+
+I<< (contributed by Stephen Thirlwall (sdt)) >>
+
+If you use Template Toolkit and want to dump your variables using Data::Printer,
+install the L<Template::Plugin::DataPrinter> module and load it in your template:
+
+   [% USE DataPrinter %]
+
+The provided methods match those of C<Template::Plugin::Dumper>:
+
+   ansi-colored dump of the data structure in "myvar":
+   [% DataPrinter.dump( myvar ) %]
+
+   html-formatted, colored dump of the same data structure:
+   [% DataPrinter.dump_html( myvar ) %]
+
+The module allows several customization options, even letting you load it as a
+complete drop-in replacement for Template::Plugin::Dumper so you don't even have
+to change your previous templates!
+
 =head2 Unified interface for Data::Printer and other debug formatters
 
 I<< (contributed by Kevin McGrath (catlgrep)) >>
@@ -1631,6 +1920,13 @@ data structures too!
 
 You can check you L<dip>'s own documentation for more information and options.
 
+=head2 Sample output for color fine-tuning
+
+I<< (contributed by Yanick Champoux (yanick)) >>
+
+The "examples/try_me.pl" file included in this distribution has a sample
+dump with a complex data structure to let you quickly test color schemes.
+
 
 =head1 BUGS
 
@@ -1666,6 +1962,8 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Allan Whiteford
 
+=item * Andy Bach
+
 =item * Árpád Szász
 
 =item * brian d foy
@@ -1690,10 +1988,14 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Fitz Elliott
 
+=item * Ivan Bessarabov (bessarabv)
+
 =item * J Mash
 
 =item * Jesse Luehrs (doy)
 
+=item * Joel Berger (jberger)
+
 =item * Kartik Thakore (kthakore)
 
 =item * Kevin Dawson (bowtie)
@@ -1706,16 +2008,26 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Matt S. Trout (mst)
 
+=item * Maxim Vuets
+
 =item * Mike Doherty (doherty)
 
 =item * Paul Evans (LeoNerd)
 
 =item * Przemysław Wesołek (jest)
 
+=item * Rebecca Turner (iarna)
+
+=item * Rob Hoelz (hoelzro)
+
 =item * Sebastian Willing (Sewi)
 
 =item * Sergey Aleynikov (randir)
 
+=item * Stanislaw Pusep (syp)
+
+=item * Stephen Thirlwall (sdt)
+
 =item * sugyan
 
 =item * Tatsuhiko Miyagawa (miyagawa)
@@ -1726,6 +2038,8 @@ with patches, bug reports, wishlists, comments and tests. They are
 
 =item * Wesley Dal`Col (blabos)
 
+=item * Yanick Champoux (yanick)
+
 =back
 
 If I missed your name, please drop me a line!
