@@ -1,7 +1,8 @@
 package Data::Printer::Filter;
 use strict;
 use warnings;
-use Clone qw(clone);
+use Clone::PP qw(clone);
+require Carp;
 require Data::Printer;
 
 my %_filters_for = ();
@@ -15,6 +16,9 @@ sub import {
     my $filter = sub {
         my ($type, $code) = @_;
 
+        Carp::croak "syntax: filter 'Class', sub { ... }"
+          unless $type and $code and ref $code eq 'CODE';
+
         unshift @{ $_filters_for{$id}{$type} }, sub {
             my ($item, $p) = @_;
 
@@ -48,7 +52,6 @@ sub import {
 
     my $imported = sub (\[@$%&];%) {
         my ($item, $p) = @_;
-        require Data::Printer;
         return Data::Printer::p( $item, %properties );
     };
 
@@ -102,10 +105,10 @@ Create your filter module:
 
       return $ref->some_method;   # or whatever
 
-      # see L</HELPER FUNCTIONS> below for
+      # see 'HELPER FUNCTIONS' below for
       # customization options, including
       # proper indentation.
-  }
+  };
 
   1;
 
